/*******************************
 *          __                   
 *         /\ \                  
 *  __  _  \_\ \     __    ___   
 * /\ \/'\ /'_' \  /'__'\ /'___\ 
 * \/>  <//\ \ \ \/\  __//\ \__/ 
 *  /\_/\_\ \___,_\ \____\ \____\
 *  \//\/_/\/__,_ /\/____/\/____/
 * 
 * Author: 	declis
 * E-Mail:	declis@xdec.de
 ********************************/ 

#include <msp430g2553.h>
#include "typedef.h"
#include "delay.h"
#include "lib_sound.h"
			
const uint snd_pattern[][PATTERN_SIZE]={
Cs3,Ds3,0,Cs3,Ds3,0,Cs3,Ds3,0,Cs3,Ds3,0,0,Cs3,Ds3,Cs3,					// 00
D3,E3,0,D3,E3,0,D3,E3,0,D3,E3,0,0,D3,E3,D3,								// 01
C3,C4,0,C3,C4,0,C3,C4,0,C3,C4,0,0,C3,C4,C3,								// 02
C3,0,C3,0,C3,0,C3,0,C3,0,C4,0,0,C4,0,C4,								// 03
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,										// 04 filler
Cs3,0,Cs3,0,Cs3,0,Cs3,0,Cs3,0,C4,0,0,C4,0,C4,							// 05
Cs3,Cs3,0,0,D3,0,G4,Fs4,Cs3,Cs3,0,0,D3,Fs3,Fs4,Cs3,						// 06
Cs3,Cs3,0,0,D3,G4,Fs4,Cs3,Cs3,F3,Fs3,F3,C4,F3,Fs3,0,					// 07
Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,Cs3,		// 08
G3,G4,As4,G3,G4,As4,G3,G4,As4,G3,G4,As4,G3,G4,As4,As3,					// 09
0,0,As3,0,As3,0,As3,0,As3,0,As3,As3,As3,As3,As3,As3,					// 10
Ds3,Ds4,Gs4,Ds3,Ds4,Gs4,Ds3,Ds4,Gs4,Ds3,Ds4,Gs4,Ds3,Ds4,Gs4,Cs4,		// 11
C3,C4,As4,C3,C4,As4,C3,C4,As4,C3,C4,As4,C3,C4,As4,Gs4,					// 12
C3,C4,As4,0,C3,C4,As4,0,C3,C4,As4,0,C3,C4,As4,0,						// 13
Ds3,Ds4,Gs4,0,Ds3,Ds4,Gs4,0,Ds3,Ds4,Gs4,0,Ds3,Ds4,Gs4,0,				// 14
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0											// 15 filler
};

// array for playing the patterns in snd_pattern array
const uchar snd_sequencer[]={
15,15,8,8,1,0,0,1,5,2,2,2,3,2,5,2,0,2,2,1,2,2,
5,3,6,7,6,7,7,6,6,5,6,7,6,5,10,9,9,10,9,9,9,5,
9,10,11,12,12,9,12,10,11,11,12,9,11,11,9,11,9,14,11,12,11,12,
5,13,12,14,11,13,12,14,11,12,13,12,11,12,11,14,9,13,12,12,11,12,
11,13,11,12,14,11,9,3
};

uint snd_ms_tick;
uchar snd_loop,snd_pattern_pos,snd_notes_pos,snd_sequencer_size,snd_cut_off;

void init_sound(uint bmp, uchar mode)
{
	P1DIR|=SPEAKER;		// output
	P1SEL|=SPEAKER;		// TA0.0 option (PWM output)
	P1OUT&=~SPEAKER;
	
	// init. ACLK (32kHz crystal)
	BCSCTL3|=XCAP_3;	// CL=~12.5pF
	do
	{
		IFG1&=~OFIFG;	// IFG1=Interrupt Flag Register 1
		wait_ms(1);		// OFIFG=Oscillator Fault Interrupt Flag
	}
	while(IFG1&OFIFG);
	
	snd_cut_off=0;		// var for note dead time
	snd_pattern_pos=0;	// pattern position
	snd_notes_pos=0;	// notes position (current pattern)
	if(mode) snd_loop=1;// play in loop
	else snd_loop=0;	// play only once
	
	snd_sequencer_size=sizeof(snd_sequencer);	// get sequencer size
	snd_ms_tick=60000/(TICKS_PER_BEAT*bmp);		// note duration
	
	// TimerA0 for NOTE FREQUENCY
    TA0CTL|=TASSEL_2+MC_0+ID_3;			// SMCLK=16MHz, Timer Div: 8, timer in stop mode		
    TA0CCR0=snd_pattern[snd_sequencer[snd_pattern_pos]][snd_notes_pos++];	// first note frequency
    TA0CCR1=TA0CCR0>>DUTY_CYCLE;
    TA0CCTL1|=OUTMOD_7;					// reset/set (PWM)
    
    // TimerA1 for NOTE DURATION (BMP)
    TA1CTL|=TASSEL_1+MC_0+ID_0;			// 32kHz crystal, no divider
    //set duration (32768*snd_ms_tick*notes_duration[snd_notes_pos++])/1000-DEAD_MS;
    TA1CCR0=((snd_ms_tick-DEAD_MS)<<5);
    TA1CCTL0|=CCIE;
    _EINT();
}

void start_sound(void)
{
    TA0CTL|=MC_1;
    TA1CTL|=MC_1;
}

void stop_sound(void)
{
    TA0CTL&=~MC_1;
    TA1CTL&=~MC_1;
}

#pragma INTERRUPT (update_note);
#pragma vector=TIMER1_A0_VECTOR
void update_note(void) 
{
	TA1CTL&=~MC_1;				// stop duration timer
	TA1R=0;						// set count register to zero
	if(!snd_cut_off)			// DEAD TIME
	{
		TA0CCR0=0;				// stop note frequency timer (a ZERO will stop the timer)
		TA1CCR0=DEAD_MS<<5;		// set dead time
		snd_cut_off=1;			// will be used after the dead time is over (load next note)
	}
	else 						// NOTE TIME
	{
		if(snd_pattern_pos<snd_sequencer_size)
		{
			// set note
			TA0CCR0=snd_pattern[snd_sequencer[snd_pattern_pos]][snd_notes_pos++];
			TA0CCR1=TA0CCR0>>DUTY_CYCLE;
			
			// set duration (32768*snd_ms_tick*notes_duration[snd_notes_pos++])/1000-DEAD_MS;
			//TA1CCR0=((snd_ms_tick*notes_duration[snd_notes_pos++])<<5)-DEAD_MS;
			TA1CCR0=((snd_ms_tick-DEAD_MS)<<5);
			if(snd_notes_pos==PATTERN_SIZE) 
			{	
				snd_pattern_pos++;	// next pattern
				snd_notes_pos=0;	// set note position to first note in pattern
				if(snd_pattern_pos==snd_sequencer_size&&snd_loop)	// loop mode, start again at first pattern
					snd_pattern_pos=0;
			}
		}
		else stop_sound();	
		snd_cut_off=0;			// for dead time, see if(!snd_cut_off)
	}
	TA1CTL|=MC_1;				// start timer again
}
