/*******************************
 *          __                   
 *         /\ \                  
 *  __  _  \_\ \     __    ___   
 * /\ \/'\ /'_' \  /'__'\ /'___\ 
 * \/>  <//\ \ \ \/\  __//\ \__/ 
 *  /\_/\_\ \___,_\ \____\ \____\
 *  \//\/_/\/__,_ /\/____/\/____/
 * 
 * Author: 	declis
 * E-Mail:	declis@xdec.de
 ********************************/ 

#include <msp430g2553.h>
#include "typedef.h"
#include "init_display.h"
#include "lib_math.h"
#include "lib_RNG.h"
#include "lib_lcd.h"
#include "delay.h"
#include "lib_sound.h"
#include "const_data.h"

#define C_BORDER	0x0005	// Border Color
#define BORDER_S	45		// Border start
#define BORDER_E	275		// Border end

#define META_SIZE	3		// amount of metaballs
#define S_NUM		90		// Don't go higher with amount of 3d dots

// some help functions for smaller code size
void clean_area(uint);
void init_star(uint);
void get_star_pos(uint,uint,uint);
void draw_lissa_sin_sin(uint,uint,uint,uint,uint);
void scene_wireframe(uint,uint,uint,uint,uint,uint,uint,uint);
void scene_dot_cube(uint,uint,uint,uint);

struct point3d_schar dots3d[S_NUM];
struct meta_b m_balls[META_SIZE];
struct point3d_sint dot3d[2];

// space between chars
uchar space_char=1;		

// help vars
uint x3;
sint x=0,y=0,x1=0,y1=0,x2=0,y2=0,r=0,i=0,z=0,x4=0;
uchar b=0;

void main(void)
{
	WDTCTL=WDTPW+WDTHOLD;		// stop watchdog
	BCSCTL1=CALBC1_16MHZ;		// SMCLK=~16MHz  
  	DCOCTL=CALDCO_16MHZ;

  	init_ADC(1,BIT1);			// init. ADC (RNG)
  	init_USCI();				// init. USCI (SPI)
  	init_LCD(C_BLACK);			// init. Display
  	
  	// METABALLS INIT ------------------------------------------------
	m_balls[0].x=10; m_balls[0].y=20; m_balls[0].direction=4;
	m_balls[1].x=30; m_balls[1].y=50; m_balls[1].direction=2;
	m_balls[2].x=45; m_balls[2].y=25; m_balls[2].direction=3;
	// ---------------------------------------------------------------
	
	qran_seed=(ulong)get_seed(6);	// init. pseudo RNG with a seed value from real RNG
  	init_sound(95,ONCE);			// init. sound
	start_sound();					// start sound
	
  	// FADE IN HELLO REVISION ----------------------------------------
  	while(x<64)
  	{
  		draw_string(85,160,string_00,set_RGB16(0,x++,0),TRANSP,0,0);
  		wait_ms(60);
  	}
	wait_ms(800);
  	// --------------------------------------------------------------- 	
  	
  	// CHAOS RANDOM BINARY CODE --------------------------------------
  	x=0;
	while(x++<2300)
		draw_char_text_mode(get_rand_pseudo(0,39),get_rand_pseudo(0,51),get_rand_pseudo(0,1)+48,
								set_RGB16(0,get_rand_pseudo(0,63),0),C_BLACK,0,0);
	// ---------------------------------------------------------------	
	
	// CREATE BORDERS ------------------------------------------------
	fill_display(lcd_width,lcd_height,C_BORDER);
	x=lcd_height/2;y=lcd_height/2;	// open up at the middle of display
	while(x!=45)
	{
		fill_rect(0,lcd_width-1,x,x,C_DARK_BLUE);
		fill_rect(0,lcd_width-1,x-1,x-1,C_BORDER+10);	// top line
		x--;
		fill_rect(0,lcd_width-1,y,y,C_DARK_BLUE);
		fill_rect(0,lcd_width-1,y+1,y+1,C_BORDER+10);	// bottom line
		y++;
		wait_ms(5);
	}
	fill_rect(0,lcd_width-1,40,40,C_BORDER+10);		// 2nd line
	fill_rect(0,lcd_width-1,280,280,C_BORDER+10);	// 2nd line
	// ---------------------------------------------------------------	
	
	wait_ms(500);
	
	// XDEC Y-ROTATE -------------------------------------------------
	x3=0;x1=300;r=60;
	while(r<200)
	{
		for(x2=0,i=0;x2<4;x2++,i-=90)	// rotate letter dots, y-axis with -90 offset for every letter
			rotate_3dp_array(obj_3d,dots3d,letter_dots_pos[x2][0],letter_dots_pos[x2][1],0,y+i,0,100,r);
		for(x2=0;x2<4;x2++)				// draw letters
			draw_3d_obj(x1,y1,dots3d,obj_edges,letter_edges_pos[x2][0],letter_edges_pos[x2][1],C_FUCHSIA);
		wait_ms(10);	
		for(x2=0;x2<4;x2++)				// clean letters
			draw_3d_obj(x1,y1,dots3d,obj_edges,letter_edges_pos[x2][0],letter_edges_pos[x2][1],C_DARK_BLUE);
		y+=8;					// y-angle
		y1=(get_sin(x3)>>3)+160;
		x3+=5;					// sine index
		if(x1>120) x1-=2;		// move in from right
		if(x3>700) r++;			// get smaller
	}
	// ---------------------------------------------------------------	
	
	// ROTATE TWO CUBES, MOVE IN FROM RIGHT AND LEFT 
	x=-60;y=160;z=0;x1=lcd_width+60;x3=0;
	x2=0;y2=lcd_width-1;
	while(x2<lcd_width)
	{
		draw_pixel_xy(x2,(get_sin(x2*4)>>5)+90,C_RED,0,0);
		draw_pixel_xy(x2,(get_cos(x2*6)>>5)+95,C_GREEN,0,0);
		draw_pixel_xy(x2,(get_sin(x2*5)>>5)+225,C_RED,0,0);
		draw_pixel_xy(x2,(get_cos(x2*3)>>5)+230,C_GREEN,0,0);
		x2++;
	}
	while(x!=110&&z!=720)
	{	
		x+=2;x1-=2;z+=10;x3+=5;
		rotate_3dp_array(obj_3d,dots3d,CUBE_DOTS_S,CUBE_DOTS_E,x3,z,0,100,130);
		draw_3d_obj(x,y,dots3d,obj_edges,CUBE_EDGES_S,CUBE_EDGES_E,C_RED);			// draw red cube (from left)
		draw_3d_obj(x1,y,dots3d,obj_edges,CUBE_EDGES_S,CUBE_EDGES_E,C_LIME);		// draw green cube (from right)
		wait_ms(2);
		draw_3d_obj(x,y,dots3d,obj_edges,CUBE_EDGES_S,CUBE_EDGES_E,C_DARK_BLUE);	// delete cube		
		draw_3d_obj(x1,y,dots3d,obj_edges,CUBE_EDGES_S,CUBE_EDGES_E,C_DARK_BLUE);	// delete cube
	}
	// ---------------------------------------------------------------	
	
	// DRAW REVISION LOGO AND METABALLS ------------------------------
	draw_bmap_8bit_v(60,130,rev_logo,64,64,C_DARK_BLUE,C_RED,0,0);
	draw_metaballs(130,130,m_balls,META_SIZE,2000,C_LIME,C_DARK_BLUE);
	// ---------------------------------------------------------------
	
	// METABALLS PHYSICS ---------------------------------------------
	x=2000; // balls size
	while(x<15000)
	{
		draw_metaballs(130,130,m_balls,META_SIZE,x,C_LIME,C_DARK_BLUE);
		x+=700;
	}
	y1=0;y2=2;x=4;y=15000;x3=META_SIZE;
	while(y2)	// smaller / bigger effect
	{
		draw_metaballs(130,130,m_balls,META_SIZE,y,C_LIME,C_DARK_BLUE);
		meta_ball_physics(m_balls,META_SIZE,x,10,130,130);
		if(y<=2000) 
		{
			y1=1;	
			y2--;	// duration
			x++;	// pos_change
		}
		if(y>=15000) y1=0;
		if(!y1) y-=500;	// bigger
		else y+=500;	// smaller
	}
	// ----------------------------------------------------------------
	
	// DEAD PIXEL FACE ------------------------------------------------
	fill_rect(130,130+64,130,130+64,C_GREY);		// pixel face
	draw_char(74,70,'x',C_RED,TRANSP,1,1);
	draw_char(84,70,'X',C_RED,TRANSP,1,1);
	draw_char(52,85,'_',C_RED,TRANSP,2,1);
	draw_line(228,163,190,170,C_GREY,0,0);
	draw_string(45,100,string_02,C_RED,TRANSP,0,1); // dead pixel string
	// ----------------------------------------------------------------
	
	wait_ms(3000);
	clean_area(C_DARK_BLUE);
	
	// RAINBOW LISSA FIGURES ------------------------------------------
	x3=380;r=0;y=130;
	while(y>-180)
	{
		draw_lissa_sin_sin(y+20,190,3,7,wavelen2rgb(x3));
		draw_lissa_sin_sin(y,130,2,3,wavelen2rgb(x3));

		draw_lissa_sin_sin(y+20,190,3,7,C_DARK_BLUE);
		draw_lissa_sin_sin(y,130,2,3,C_DARK_BLUE);

		r+=5;				// var for movement effect
		x3+=4;				// effect duration
		if(x3>800) y-=4;	// for x-pos, move to left
	}
	// ---------------------------------------------------------------	
	
	clean_area(C_DARK_BLUE);
	
	// SLOW LISSA DRAWING FOR TWISTER --------------------------------
	x3=0;
	while(x3<65500)
	{
		draw_pixel_xy((get_sin(x3)/3)+120,(get_sin(10*x3)/3)+160,C_PURPLE,0,0);
		x3++;
		if(x3<360) wait_ms(5);
	}
	// DRAW FACE -----------------------------------------------------
	draw_bmap_8bit_v(110,100,face,91,120,C_TEAL,TRANSP,0,0);
	wait_ms(1000);
	// ---------------------------------------------------------------
	
	// TWISTER -------------------------------------------------------
	y=0;i=0;x=0;r=0;z=y;y2=0;b=0;
	while(r<190)
	{
		while(y++<140)
		{
			if(r>100) z=y>>1;	// not so heavy
			else z=y;			// heavy twist
			if(!b) y2++;		// y2 for color effect on 2 faces
			else y2--;
			if(!y2) b=0;		
			if(y2>20) b=1;
			draw_h_line(54,96,y+90,C_DARK_BLUE); 	// 31 width or 42 atm
			
			//x1=(get_sin(z+i)>>5)+75; // a small movement effect
			//x2=(get_sin(z+i+90)>>4)+75;
			//x3=(get_sin(z+i+180)>>4)+75;
			//x4=(get_sin(z+i+270)>>5)+75;
			
			// could be much much faster
			x1=(get_sin(z+i)/18)+75; // a small x-movement effect
			x2=(get_sin(z+i+90)/12)+75;
			x3=(get_sin(z+i+180)/12)+75;
			x4=(get_sin(z+i+270)/18)+75;
			if(x1<x2) draw_h_line(x1,x2,y+90,C_BLUE-y2);
			if(x2<x3) draw_h_line(x2,x3,y+90,C_BORDER+4);
			if(x3<x4) draw_h_line(x3,x4,y+90,C_BLUE-y2);
			if(x4<x1) draw_h_line(x4,x1,y+90,C_BORDER+4);
		}
		y=0;	// height of twister
		if(r<100) i+=10;	// movement speed
		if(r>100) i-=15;
		r++;	// duration
	}	
	// ---------------------------------------------------------------
	
	clean_area(C_DARK_BLUE);
	wait_ms(500);

	// DRAW HORIZONTAL SIN/COS ---------------------------------------
	y=0;
	while(y++<228)
	{
		draw_pixel_xy((get_sin(y*4)>>4)+205,y+46,C_BLUE,0,0);
		draw_pixel_xy((get_cos(y*6)>>5)+205,y+46,C_MAROON,0,0);
		draw_pixel_xy((get_sin(y*8)>>4)+215,y+46,C_PURPLE,0,0);
		wait_ms(5);
	}
	// ---------------------------------------------------------------
	
	// ROTATING 3D CUBE WITH FONT (YEAH) -----------------------------
	x3=0;x=0;y=0;z=0;i=0;x1=0;r=350;y2=-130;
	while(x3<400)
	{	
		for(x2=0,i=0;x2<4;x2++,i-=90)	// rotate letter dots
			rotate_3dp_array(obj_3d,dots3d,letter_dots_pos[x2][0],letter_dots_pos[x2][1],0,y+i,z,100,r);
		rotate_3dp_array(obj_3d,dots3d,CUBE_DOTS_S,CUBE_DOTS_E,0,y,z,100,r);			// rotate cube dots
		draw_3d_obj(x1+y2,y1,dots3d,obj_edges,CUBE_EDGES_S,CUBE_EDGES_E,C_FUCHSIA);		// draw cube
		if(x3>250)	// draw letters on cube
		{
			for(x2=0;x2<4;x2++)
				draw_3d_obj(x1+y2,y1,dots3d,obj_edges,letter_edges_pos[x2][0],letter_edges_pos[x2][1],C_LIME);
		}
		wait_ms(1);
		draw_3d_obj(x1+y2,y1,dots3d,obj_edges,CUBE_EDGES_S,CUBE_EDGES_E,C_DARK_BLUE);	// delete cube
		for(x2=0;x2<4;x2++)																// delete letters
			draw_3d_obj(x1+y2,y1,dots3d,obj_edges,letter_edges_pos[x2][0],letter_edges_pos[x2][1],C_DARK_BLUE);
		y+=7;z+=5;					// rotating angles
		x1=get_sin(x)>>3;			
		y1=160+(get_sin(3*x)>>2);	
		x+=4;						// sine wave index
		x3+=1;						// duration
		if(y2<100) y2+=2;			// move cube in from left side
		if(x3>50&&x3<200&&r>115) r-=2; 	// grow up
		if(x3>310&&r<300) r+=2; 	// shrink down
	}	
	// ---------------------------------------------------------------
	
	// ROTATING OKTA -------------------------------------------------
	scene_wireframe(OKTA_DOTS_S,OKTA_DOTS_E,OKTA_EDGES_S,OKTA_EDGES_E,C_RED,C_DARK_BLUE,10,86);
	// ---------------------------------------------------------------
	
	// ROTATING IKO -------------------------------------------------
	scene_wireframe(IKO_DOTS_S,IKO_DOTS_E,IKO_EDGES_S,IKO_EDGES_E,C_GREEN,C_DARK_BLUE,2,85);
	// ---------------------------------------------------------------
	
	// ROTATING DODE -------------------------------------------------
	scene_wireframe(DODE_DOTS_S,DODE_DOTS_E,DODE_EDGES_S,DODE_EDGES_E,C_BLUE,C_DARK_BLUE,2,125);
	// ---------------------------------------------------------------
	
	clean_area(C_DARK_BLUE);
	wait_ms(500);
	
	// RAINBOW SCROLLER ----------------------------------------------
	space_char=7;
	x1=120;y=0;y1=0;y2=0;x3=0;
	for(i=0;i<S_NUM;i++)	// star init.
		init_star(i);
		
	while(x1>-1620)
	{
		for(i=0;i<S_NUM;i++)
		{
			get_star_pos(i,120,160);	// calculate x,y position
			if(x>=lcd_width||x<0||y<46||y>272||dots3d[i].z<=0)
				init_star(i); 	// outside display, init. new star
			else 
			{
				r=31-((dots3d[i].z<<5)>>7);	// fade in
				if(dots3d[i].z<40)			// bigger star
					draw_pixel_xy(x,y,set_RGB16(r,r,r),2,2);
				else draw_pixel_xy(x,y,set_RGB16(r,r,r),1,1);
			}
		}
		
		y2=(get_sin(x3)>>4)+80;
		draw_string_sin(x1,y2,scroller_01,x3,8,wavelen2rgb(y1++),TRANSP,3,1);
		wait_ms(5);
		// clean stars
		for(i=0;i<S_NUM;i++)
		{
			get_star_pos(i,120,160);
			if(x<lcd_width&&x>=0&&y>45&&y<273&&dots3d[i].z>=0)
			{
				if(dots3d[i].z<40)
					draw_pixel_xy(x,y,C_DARK_BLUE,2,2);
				else draw_pixel_xy(x,y,C_DARK_BLUE,1,1);	
			}
		}
		// move stars
		for(i=0;i<S_NUM;i++)
			dots3d[i].z+=3;
		draw_string_sin(x1,y2,scroller_01,x3,8,C_DARK_BLUE,TRANSP,3,1);
		x1-=4;	// string x-pos speed
		x3+=6; 
	}
	// ---------------------------------------------------------------
	
	// ANIMATED LISSA WITH CHARS -------------------------------------
	x3=0;r=0;i=1;
	while(x3<150)
	{
		for(x=0;x<300;x+=6)
		{
			x1=(get_sin(x)/5)+57;
			y1=(get_sin(i*x+r)/5)+160;
			draw_char(x1,y1,'x',C_TEAL,TRANSP,1,0);
		}
		for(x=0;x<300;x+=6)
		{
			x1=(get_sin(x)/5)+57;
			y1=(get_sin(i*x+r)/5)+160;
			draw_char(x1,y1,'x',C_DARK_BLUE,TRANSP,1,0);
		}
		r+=16;
		x3++;
		if(x3>50&&x3<52) i++;
		if(x3>100&&x3<102) i++;
	}
	// ---------------------------------------------------------------
	
	clean_area(C_DARK_BLUE);
	
	// DOT CUBE ------------------------------------------------------
	x3=0;i=3;x1=0;y1=-30;
	while(x3<215)
	{
		x2=(get_sin(r)>>2)+y1;
		scene_dot_cube(x2,160,i,C_FUCHSIA);
		wait_ms(3);
		scene_dot_cube(x2,160,i,C_DARK_BLUE);
		r+=10;
		x3++;		
		if(!x1&&TA0CCR0==As4) i+=2;
		if(x1&&TA0CCR0!=As4) i-=1;
		if(i>9) x1=1;
		if(i<5)	x1=0;
		if(y1<120) y1+=2;
		if(x3>185) y1+=7;
	}
	// ---------------------------------------------------------------
	
	clean_area(C_BLACK);
	wait_ms(500);
	
	// THE END - INFO STRINGS ----------------------------------------
	space_char=1;
	for(x=0;x<3;x++)
		draw_string(8,x*12+4,string_05[x],C_TEAL,TRANSP,0,0);
	draw_string(4,97,string_06,C_TEAL,TRANSP,4,2);
	// ---------------------------------------------------------------
	
	// THE END - STARFIELD EFFECT AND SCROLLER -----------------------
	x3=0;x2=300;y1=0;y2=0;space_char=10;
	
	for(i=0;i<S_NUM;i++)	// star init.
		init_star(i);
	
	while(TA0CTL&0x10)
	{
		y1=(get_sin(x2<<1)>>3)+120;
		y2=(get_sin(x2-45)>>3)+160;
		// draw stars
		for(i=0;i<S_NUM;i++)
		{
			get_star_pos(i,y1,y2);	// calculate x,y position
			if(x>=lcd_width||x<0||y<46||y>272||dots3d[i].z<=0)
				init_star(i); 	// outside display, init. new star
			else 
			{
				r=31-((dots3d[i].z<<5)>>7);	// fade in
				if(dots3d[i].z<40)			// bigger star
					draw_pixel_xy(x,y,set_RGB16(0,r,r),2,2);
				else draw_pixel_xy(x,y,set_RGB16(0,r,r),1,1);
			}
		}
		draw_string_sin(x2,75,scroller_03,x3,6,C_LIME,TRANSP,2,1);
		wait_ms(5);
		
		// clean stars
		for(i=0;i<S_NUM;i++)
		{
			get_star_pos(i,y1,y2);
			if(x<lcd_width&&x>=0&&y>45&&y<273&&dots3d[i].z>=0)
			{
				if(dots3d[i].z<40)
					draw_pixel_xy(x,y,C_BLACK,2,2);
				else draw_pixel_xy(x,y,C_BLACK,1,1);	
			}
		}
		// move stars
		for(i=0;i<S_NUM;i++)
			dots3d[i].z-=2;
		draw_string_sin(x2,75,scroller_03,x3,6,C_BLACK,TRANSP,2,1);
		x2-=3;
		x3+=8;
	}	
	// ---------------------------------------------------------------
	space_char=1;
	fill_display(lcd_width,lcd_height,C_BLACK);
	draw_string(30,20,string_07,C_WHITE,TRANSP,2,2);
	draw_string(35,50,string_08,C_WHITE,TRANSP,2,2);
	draw_string(17,80,string_09,C_WHITE,TRANSP,2,2);
	wait_ms(1000);
	fill_display(lcd_width,lcd_height,C_BLACK);
	while(1);
	
}

void get_star_pos(uint pos, uint x_off, uint y_off)
{
	x=(dots3d[pos].x<<7)/dots3d[pos].z+x_off;
	y=(dots3d[pos].y<<7)/dots3d[pos].z+y_off;
}

void init_star(uint pos)
{
	dots3d[pos].x=get_rand_pseudo(-80,80); 
	dots3d[pos].y=get_rand_pseudo(-80,80); 
	dots3d[pos].z=get_rand_pseudo(20,127); 
}

void clean_area(uint color)
{
	x=46;
	while(x++<274)
		fill_rect(0,239,x,x,color);
}

void draw_lissa_sin_sin(uint x_pos, uint y_pos, uint x_fac, uint y_fac, uint color)
{
	for(x=0;x<180;x+=2)
	{
		x1=(((slong)get_sin(x)*get_sin(x_fac*x))>>9)+x_pos;
		y1=(((slong)get_sin(x+r)*get_sin(y_fac*x+r))>>10)+y_pos;
		draw_pixel_xy(x1,y1,color,2,1);
	}
}

void scene_wireframe(uint dot_start, uint dot_end, uint edge_start, uint edge_end, uint F_COL, uint B_COL, uint time, uint size)
{
	x3=0;i=0;r=350;y2=100;
	while(x3<400)
	{													
		rotate_3dp_array(obj_3d,dots3d,dot_start,dot_end,z,y,z,100,r);		// rotate
		draw_3d_obj(x1+y2,y1,dots3d,obj_edges,edge_start,edge_end,F_COL);	// draw
		wait_ms(time);
		draw_3d_obj(x1+y2,y1,dots3d,obj_edges,edge_start,edge_end,B_COL);	// delete

		y+=7;z+=5;					// rotating angles
		x1=get_sin(x)>>3;			// x-pos lissa
		y1=160+(get_sin(3*x)>>2);	// y-pos lissa
		x+=4;						// sine wave index
		x3+=1;						// var for animation length
		if(x3>50&&x3<200&&r>size) r-=2; 	// grow up
		if(x3>280&&r<300) r+=2; 		// shrink down
	}
}

// dist = distance between dots (nice effect with music)
void scene_dot_cube(uint x_pos, uint y_pos, uint dist, uint color)
{
	for(z=-2;z<=2;z++)
	{
		for(y=-2;y<=2;y++)
		{	
			for(x=-2;x<=2;x++)
			{
				dot3d[0].x=x*dist;
				dot3d[0].y=y*dist;
				dot3d[0].z=z*dist;
				rotate_3dp(dot3d[0],&dot3d[1],r,r,0,100,50);
				draw_pixel_xy(dot3d[1].x+x_pos,dot3d[1].y+y_pos,color,1,1);
			}
		}
	}	
}
